/*
* $Header: /home/dto/octal/RCS/delay.c,v 1.4 2000/07/19 08:00:06 dto Exp dto $
* (C) 2000 David O'Toole $Date: 2000/07/19 08:00:06 $
* $Revision: 1.4 $
* 
* DELAY.C
* A (very) simple delay effect for OCTAL. 
* Currently only in mono.  
*
* This software is distributed under the terms of the
* GNU General Public License (GPL). Read the included file
* COPYING for more information. 
* 
*/

static const char rcsid[]="$Id: delay.c,v 1.4 2000/07/19 08:00:06 dto Exp dto $";

#include "util.h"    /* utility functions for music  */
#include "machine.h" /* machine interface            */

#include <stdlib.h>
#include <string.h>

enum {ix_length, ix_feedback};  /* indexes of parameters in an event */
enum {my_track = 1 };           /* always 1 with effects             */

param_spec my_params[] = { 
  {
    small,             /* how to display it in pattern view */ 
    slider,            /* how to control it in control box  */ 
    "len",
    "delay length (in blocks)",
    1,                /* minimum           */
    8,                /* maximum           */ 
    3                 /* default           */ 
  },

  {
    small,            /* show as two hex digits, 00-FE */
    slider,           /* control type is slider        */ 
    "fb", 
    "Feedback level (0xFE=100%)",
    0x00, 
    0xFE, 
    0x10,             /* low default feedback level    */ 
  }
};


/* note: parameter 0 is currently ignored... will be fixed later */

typedef struct _my_state { 
  samp* blockbuf;
  float feedback_factor;
} my_state; 

 
int ox_init(machine_type* t) {
  
  t->long_name        = "Simple Mono Delay Machine"; 
  t->short_name       = "mdelay"; 
  t->max_tracks       = 1; 
  t->input_channels   = 1; 
  t->output_channels  = 1;   
  t->num_params       = 2;

  t->param_specs      = my_params; /* give it the address of my array of param structs */ 

  return(1); 
}

 
void ox_create(machine* m) 
{
  my_state* s;

  s = (my_state*) malloc(sizeof(my_state));

  s->feedback_factor = ((float)0x10) / 255.0 ;     
  s->blockbuf = (float*) m->pkg->alloc(OX_SAMPLING_RATE * sizeof(float));
  memset(s->blockbuf, 0, OX_SAMPLING_RATE * sizeof(samp));

  m->state = (void*)s;  
}


void ox_destroy(machine* m) 
{
  m->pkg->free( ((my_state*)(m->state)) -> blockbuf);
  m->pkg->free(m->state);
  m->state = 0;
}


void ox_update(machine* m, int track, int param_index, param value) 
{  
  my_state *s = (my_state*)m->state; 
  
  switch (param_index) {
  case ix_length: 
    break; /* not yet implemented */ 
  case ix_feedback:
    s->feedback_factor =  ((float)value) / 255.0;   
    break; 
  }
} 


/* write to desc a string describing a given parameter value... this
   is used for interactive tweaking of parameters, giving the user
   feedback on what's happening when he/she pulls a slider...  
 */

void ox_desc(char* dest, int which_param, param value) {
  float x;
  int percent; 

  switch (which_param) {
  
  case ix_length: 
    sprintf(dest, "not implemented yet");  
    break; 

  case ix_feedback: 
    x = ((float)value) / 255.0;  /* scale from 0x00..0xFE to floating point 0..1 */ 
    percent = (int)(x * 100);
    sprintf(dest, "%d%%", percent); 
    break;

  default: 
    sprintf(dest, "ERROR"); 
  };
}


/* ox_work should actually do the job of doing output. return 1 if you generated 
 * something, 0 if not
 */

int ox_work(machine* m, int block_size) {

 my_state* s;  
 samp* mout;              /* mono out */ 
 samp* min;               /* mono in  */ 
 int i;

 s = (my_state*)m->state;       /* grab our state data address */ 
 mout = m->lout;                   
 min = m->lin; 

 /* 1. first apply the feedback decay to the existing buffer */
 /* 2. then add the current input signal to that buffer      */ 
 /* 3. output the buffer */ 

 for (i=0; i < block_size; i++) {
   s->blockbuf[i] *= s->feedback_factor;
   s->blockbuf[i] += min[i];
   mout[i] = s->blockbuf[i];
 }

 return (1);
}

void ox_track(machine* m, int change) { 
  /* leave this blank for effects, or if you don't have polyphony */ 
}

/* End $Source: /home/dto/octal/RCS/delay.c,v $ */
