/*
* $Header: /home/dto/octal/RCS/engine.h,v 1.3 2000/07/19 08:00:06 dto Exp dto $
* (C) 2000 David O'Toole $Date: 2000/07/19 08:00:06 $
* $Revision: 1.3 $
* 
* Header file for OCTAL's signal network engine. 
*
* This software is distributed under the terms of the
* GNU General Public License (GPL). Read the included file
* COPYING for more information. 
* 
*
*/

#ifndef _engine_h_
#define _engine_h_ "@(#)"

#include "machine.h"

enum {max_machines = 128 };    /* per signal network               */ 
enum {master = 0};
enum _sig_type {en_nosignal = 0, en_normal=1, en_aux}; /* types of signals in the network  */ 
typedef enum _sig_type sig_type; 

struct _engine {
  machine *machines[max_machines];
  sig_type signals[max_machines][max_machines]; 

  samp lmix[output_buffer_size]; /* where temporary sound output is stored */ 
  samp rmix[output_buffer_size];

  int num_machines; 
  long global_frame; 
  long tick;      /* current position in song             */ 
  int tickpos;    /* current position within current tick */
 
  int tick_size;  /* derived from tpb and bpm                        */ 
  int block_size; /* set to be small enough for low-latency          */
  int tpb;        /* ticks per beat                                  */ 
  int bpm;        /* beats per minute                                */ 
};

typedef struct _engine engine; 

/* some function prototypes */ 

engine* create_engine(void);
void set_timings(engine* e, int block_size, int tpb, int bpm);
int connect_machines(engine* e, sig_type t, int source, int sink);     /* machine number */ 
void disconnect_machines(engine* e, int source, int sink); 

/* add_machine is an adopter; it takes ownership of the machine's memory allocation */ 
/* therefore, remove_machine will ensure its deletion (and will call ox_destroy)    */

int add_machine(engine* e, machine* m);  /* returns index of machine's new home */ 
void remove_machine(engine* e, int m);   
machine* get_machine(engine* e, int m);
void do_block(engine* e, int m);                
int is_present(engine* e, int which_machine);  /* is there a machine there? */ 
int is_connected(engine* e, int source, int sink);

/* some mixing functions */ 

void add_copy(samp* dest, samp* src, int size); 
void bad_scale(samp* buf, int size, int numsignals); /* this is kind of yucky */ 

#endif /*_engine_h_*/

/* $Source: /home/dto/octal/RCS/engine.h,v $ */
