/*
* $Header: /home/dto/octal/RCS/engine_quux.c,v 1.1 2000/07/19 08:00:06 dto Exp dto $
* (C) 2000 David O'Toole $Date: 2000/07/19 08:00:06 $
*
* engine digraph analysis / compilation 
* 
* $Revision: 1.1 $
* This software is distributed under the terms of the
* GNU General Public License (GPL). Read the included file
* COPYING for more information. 
*
*/
static const char rcsid[]="$Id: engine_quux.c,v 1.1 2000/07/19 08:00:06 dto Exp dto $";

#include "engine_quux.h"
#include <string.h>

/* this is the internal table used to dispatch quux functions */ 

void (*engine_quux_table[])(engine*,int,int) = {ex_execute,
						ex_add,
						ex_remix,
						ex_clear};

enum { default_max_quuxes = 128 };

engine_quux* create_engine_quux(int opcode, int a, int b)
{
  engine_quux* q;

  q = malloc(sizeof(engine_quux));
  q->opcode = opcode;
  q->a = a;
  q->b = b;
  return q; 
}


/* create a default program with no quuxes */ 

engine_program* create_engine_program()
{
  engine_program* p;

  p = malloc(sizeof(engine_program));
  p->quuxes = 0;
  p->num_quuxes = 0; 
}

void destroy_engine_program(engine_program* p) 
{
  int i; 

  for (i = 0; i < p->num_quuxes; i++)
    free(p->quuxes[i]);
  free(p->quuxes);
  free(p);
}



/* add a quux to the end of the program, allocating more space if neccessary.
 * this function takes ownership of the quux you pass in
 */ 

void append_quux(engine_program* p, engine_quux* q)
{
  engine_quux** t;
  int index;

  if (p->quuxes == 0) {       /* blank program */ 
    p->quuxes = malloc( default_max_quuxes * sizeof(engine_quux*));
    p->max_quuxes = default_max_quuxes; 
  }

  index = p->num_quuxes++;

  if (index >= p->max_quuxes) {
    p->max_quuxes *= 2;             /* grow the array of pointers */ 
    t = realloc(p->quuxes, p->max_quuxes * sizeof(engine_quux*));
    if (t) 
      p->quuxes = t;
    else ox_die("Memory allocation failed during append_quux().");
  }
  
  p->quuxes[index] = q;     /* move the pointer into the list of quuxes */ 
}


/* digraph compiler */ 

void compile_digraph(engine* e, int m, engine_program* p) 
{
  int i; 
  int depcount = 0;

  /* first handle any incoming signals */ 
  for (i=1; i<e->num_machines; i++) {
    if (e->signals[i][m] == en_nosignal)  /* skip ones without signals */  
      continue; 
      
    /* ok, so we have a signal coming in from i */ 
    depcount++;
    compile_digraph(e, i, p);
    }

  /* add signals */ 
  for (i=1; i<e->num_machines; i++) {
    if (e->signals[i][m] == en_nosignal)
       continue; 
    append_quux(p, create_engine_quux(ex_add_opcode, i, depcount));
  }

  /* remix */ 
  if (depcount>1)
    append_quux(p, create_engine_quux(ex_remix_opcode, depcount, depcount));

  /* finally, process the current machine */ 
  if (m != master)
    append_quux(p, create_engine_quux(ex_execute_opcode, m, 0));
  else append_quux(p, create_engine_quux(ex_terminate_opcode, 0, 0)); /* done */ 
}


/* dispatcher */ 

int execute_engine_quux(engine* e, engine_quux* q)
{
  if (q->opcode == ex_terminate_opcode) return 0; 
  if (q->opcode > 3) ox_die("Invalid opcode.");
  engine_quux_table[q->opcode] (e, q->a, q->b);
  return 1; 
}


/* the mixer opcodes */ 

void ex_execute  (engine* e, int a, int b) 
{
  printf("ex_execute %d %d \n", a, b);
}

void ex_add  (engine* e, int a, int b) 
{
  printf("ex_add %d %d \n", a, b);
}

void ex_remix  (engine* e, int a, int b) 
{
  printf("ex_remix %d %d \n", a, b);
}

void ex_clear  (engine* e, int a, int b) 
{
  printf("ex_clear %d %d \n", a, b);
}
