/*
* $Header: /home/dto/octal/RCS/engine_view.c,v 1.1 2000/07/19 08:00:06 dto Exp dto $
* (C) 2000 David O'Toole $Date: 2000/07/19 08:00:06 $
* 
* engine_view.c 
* this is the gtk+ engine viewer for OCTAL. 
*
* $Revision: 1.1 $
*
* This software is distributed under the terms of the
* GNU General Public License (GPL). Read the included file
* COPYING for more information. 
* 
* $Log: engine_view.c,v $
* Revision 1.1  2000/07/19 08:00:06  dto
* Initial revision
*
*
*
*/
#include <gtk/gtk.h>
#include "engine.h"
#include "engine_view.h"

enum {ENGINE_VIEW_SIZE = 500, ENGINE_MACHINE_SIZE = 50};

static const char rcsid[]="$Id: engine_view.c,v 1.1 2000/07/19 08:00:06 dto Exp dto $";

static GtkWidget *window = 0, *drawing = 0; 
static engine *en =0; 
static float newx, newy; /* used while dragging */ 
static int dragging = 0; /* initially false. */ 
static int drag_machine = -1; /* int id of machine being dragged */ 


void create_engine_view(engine* e) 
{
  int x; 
 
  en = e;
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "Engine View");
  drawing = gtk_drawing_area_new(); 
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawing), 
			ENGINE_VIEW_SIZE, ENGINE_VIEW_SIZE);  
  gtk_container_add(GTK_CONTAINER(window), drawing);
  
  gtk_signal_connect(GTK_OBJECT(window),
		     "delete_event",
		     GTK_SIGNAL_FUNC(delete_event_cb),
		     NULL); 
  gtk_signal_connect(GTK_OBJECT(drawing),
		     "expose_event",
		     GTK_SIGNAL_FUNC(expose_event_cb),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(drawing),
		     "button_press_event",
		     GTK_SIGNAL_FUNC(button_press_event_cb),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(drawing),
		     "button_release_event",
		     GTK_SIGNAL_FUNC(button_release_event_cb),
		     NULL);

  gtk_widget_show_all(window);

}

void destroy_engine_view() 
{

}


/* you should have the clip rectangle set already. only expose_event is using this. */ 
void draw_machine(machine* m, GtkWidget* widget) 
{
  int px, py;  /* projected into window */ 
  int size;    /* of the string, used for centering */ 
 
  px = m->x * (float)ENGINE_VIEW_SIZE; 
  py = m->y * (float)ENGINE_VIEW_SIZE; 

  gtk_draw_box(widget->style,
	       widget->window,
	       widget->state,
	       GTK_SHADOW_OUT,
	       px, py,
	       ENGINE_MACHINE_SIZE+20,
	       ENGINE_MACHINE_SIZE);
   

  size = gdk_string_width(widget->style->font,
			  m->instance_name); 

  gtk_draw_string(widget->style,
		  widget->window,
		  widget->state,
		  px+8,
		  py + (ENGINE_MACHINE_SIZE/2),
		  m->instance_name);

}


void draw_connection(int source, int sink, GtkWidget* widget)
{
 int sx, sy, kx, ky; /* source and sink line locations */ 

 sx = en->machines[source]->x * ENGINE_VIEW_SIZE + (ENGINE_MACHINE_SIZE/2+10);
 sy = en->machines[source]->y * ENGINE_VIEW_SIZE + (ENGINE_MACHINE_SIZE/2);
 kx = en->machines[sink]->x * ENGINE_VIEW_SIZE + (ENGINE_MACHINE_SIZE/2+10);
 ky = en->machines[sink]->y * ENGINE_VIEW_SIZE + (ENGINE_MACHINE_SIZE/2);	      
 
 gdk_draw_line(widget->window,
               widget->style->fg_gc[widget->state],
	       sx, sy, kx, ky);

 gtk_draw_arrow(widget->style, 
		widget->window,
		widget->state,
		GTK_SHADOW_OUT,
		GTK_ARROW_DOWN,
		TRUE,
		(sx+kx)/2 - 5, /* midpoint formula, with a tiny adjustment */ 
		(sy+ky)/2 - 5,
		10, 10);

}

/***************************************************************************/
/* callback functions for gtk+ */ 

gint delete_event_cb(GtkWidget* w, GdkEventAny* e, gpointer data) {
  gtk_main_quit();  /* REMOVE THIS LATER! */ 
  return(FALSE); 
}


/* redraw the machines. */ 
/* should this be caching more data? to avoid using messages... */ 

gboolean expose_event_cb(GtkWidget *widget, GdkEventExpose *event, gpointer data) 
{
  int i,j;  
  machine* m; 

  g_print("Hello.");
  gdk_window_clear_area(widget->window,
			event->area.x, event->area.y,
			event->area.width, event->area.height);
  gdk_gc_set_clip_rectangle(widget->style->fg_gc[widget->state],
			    &event->area);
  
  gtk_style_set_background(widget->style,
			   widget->window,
			   widget->state);

  /* now that we're set up to clip, let's draw the connections */ 
  for (i=0; i<max_machines; i++)
    for (j=0; j<max_machines; j++)
      if (is_connected(en,i,j))
	  draw_connection(i,j,widget);

  /* and then the machines */ 
  for(i=0; i<max_machines; i++) {
    m = en->machines[i]; 
    if (m) /* make sure there is a machine there */ 
      draw_machine(m, widget);
  }
 
  /* now we're done */
  gdk_gc_set_clip_rectangle(widget->style->fg_gc[widget->state], 
			    NULL); 
  return TRUE; 
}


/* start dragging... */ 
gint button_press_event_cb(GtkWidget* w, GdkEventButton* event, gpointer data) 
{
  int i, j; 
  int px, py; 
  machine* m; 

  g_print("TEST!!!!!!!!!!!"); 

  /* let's do some hit-testing to figure out what the user is touching */ 

  for(i=0; i<max_machines; i++) {
    m = en->machines[i]; 
    if (m) {
      px = m->x * (float)ENGINE_VIEW_SIZE; 
      py = m->y * (float)ENGINE_VIEW_SIZE; 
      if ( event->x > px && event->x < px+ENGINE_MACHINE_SIZE+20
	   && event->y > py && event->y < py+ENGINE_MACHINE_SIZE  ) {
	drag_machine = i; 
	goto foundit;
      }
    }
  }      

  printf("Failed."); 
  return(TRUE); /* just forget it */  
 
 foundit: 
 
  dragging = 1;      
  printf("You are touching machine %s\n", en->machines[drag_machine]->instance_name);
  return(TRUE); 
}


/* stop dragging... */ 
gint button_release_event_cb(GtkWidget* w, GdkEventButton* event, gpointer data)
{
  machine* m; 

  m = en->machines[drag_machine];
  set_visual_pos(m, newx, newy);

  dragging = 0; 
  drag_machine = -1;
}




/* End $Source: /home/dto/octal/RCS/engine_view.c,v $*/
