/*
* $Header: /home/dto/octal/RCS/machine.h,v 1.9 2000/07/19 08:00:06 dto Exp dto $
* (C) 2000 David O'Toole $Date: 2000/07/19 08:00:06 $
* $Revision: 1.9 $
* 
* stuff related to the machine interface
*
* This software is distributed under the terms of the
* GNU General Public License (GPL). Read the included file
* COPYING for more information. 
* 
*/

#ifndef _machine_h_
#define _machine_h_ "@(#)machine.h $Revision: 1.9 $"

#include <stdlib.h>

#include "octal.h"
#include "pattern.h"

#define NAME_SIZE 128
#define MAX_TRACKS 32
#define MAX_PARAMS 32

enum {output_buffer_size = sizeof(samp)*OX_SAMPLING_RATE}; 
enum {max_types = 128};

/* some special values */ 
enum {mono = 1, stereo = 2};
enum {nochange = -1, noteoff = 0};   
enum {work_quiet, work_noisy};   /* ox_work() returns these      */ 

/* symbols for format and widget type selection */
enum { note, small, large, trigger, wave };      /* format                     */ 
enum { slider };                                 /* control widget types       */

struct _machine_type; 
struct _package;

/* param_spec is used by machines to give an abstract description of
 * their parameters 
 */

typedef struct _param_spec { 
  int type;
  int widget;    
  char* name; 
  char* description; 
  data min, max, start; 

} param_spec;


typedef struct _machine {

  struct _machine_type *type;          
  struct _package *pkg; 

  long frame;  

  void *state;                        /* this is your "this" pointer */ 
  
  /* sample i/o buffers */
  samp *lin,  *rin;  
  samp *lout, *rout;   

  /* some user-interface stuff */ 
  char instance_name[NAME_SIZE];   
  float x;
  float y; 
 
  /* timing data */ 
  int num_tracks;
  int tick_size; 
  long tick_number;   
} machine;


typedef struct _machine_type { 
  
  void* so_handle;

  int         (*ox_init)(struct _machine_type*);              
  void        (*ox_create)(machine*);   
  void        (*ox_update)(machine*,int,int,param);
  int         (*ox_work)(machine*, int);   
  void        (*ox_destroy)(machine*);
  void        (*ox_desc)(char*,int,param); 
  void        (*ox_track)(machine*, int);        

  char* short_name;   
  char* long_name; 
  int max_tracks;      
  int input_channels;   
  int output_channels; 

  int num_params;
  param_spec* param_specs;  

} machine_type;


/* eventually wavetable access stuff will be in here too */ 

typedef struct _package {
  void* (*alloc)(size_t);          /* pkg_alloc        */ 
  void  (*free)(void*);
  param (*text2note)(const char*); /* pkg_text2note    */ 
  const char* (*note2text)(param); /* you get the idea */ 
  freq  (*text2freq)(const char*);   
  freq  (*note2freq)(param);         
  
  /* more to come */ 
} package; 


/*****************************************/


void init_global_pkg(void); 

int load_type_registry(const char* machine_dir);
machine_type* get_type(const char* name);
	
int load_machine_type(machine_type *t, const char *fn);
int unload_machine_type(machine_type *t);

machine* create_machine(machine_type *t);
void destroy_machine(machine *m);

/* work buffer allocator wrappers. don't use these yourself. */  
void allocate_outbuf(machine*, samp**, samp**);
void free_buf(samp**, samp**); 

/* inspector shortcuts */ 
int is_stereo_out(machine* m);
int is_stereo_in(machine* m);
int wants_input(machine* m);

/* memory functions */ 
void* ox_alloc(size_t);   /* possibly calls mlock()      */ 
void* pkg_alloc(size_t);  /* wraps ox_alloc for machines */ 
void  ox_free(void*);    /* unlocks mem and frees       */ 

/* extras, may be erased */ 
void set_visual_pos(machine* m, float x, float y);

#endif /*_machine_h_*/

/* $Source: /home/dto/octal/RCS/machine.h,v $ */












