/*
* $Header: /home/dto/octal/RCS/pattern.c,v 1.1 2000/07/19 08:00:06 dto Exp dto $
* (C) 2000 David O'Toole $Date: 2000/07/19 08:00:06 $
*
* storage for musical patterns 
* 
* $Revision: 1.1 $
* This software is distributed under the terms of the
* GNU General Public License (GPL). Read the included file
* COPYING for more information. 
*
*/
static const char rcsid[]="$Id: pattern.c,v 1.1 2000/07/19 08:00:06 dto Exp dto $";
#include "pattern.h"
#include "machine.h"
#include <string.h>

/* make a default pattern according to spec, with one track */ 

pattern* create_pattern(int params_per_track, int num_rows)
{
  pattern* p;

  p = (void*)malloc(sizeof(pattern));
  p->num_tracks = 0; 
  p->num_rows = num_rows; 
  p->params_per_track = params_per_track; 
  p->max_tracks = default_tracks; 
  p->cells = malloc(default_tracks * sizeof(param*));
  add_track(p); /* start with one */ 
  return p; 
}


void destroy_pattern(pattern* p)
{
  int i; 
  
  for (i = 0; i < p->num_tracks; i++)
    free(p->cells[i]);
  free(p->cells); 
  free(p); 
}


void add_track(pattern* p)
{
  int numcells = p->params_per_track * p->num_rows; /* params in 1 track */ 
  param** temp; 
  int i, index; 

  index = p->num_tracks++;
  p->cells[index] = malloc(numcells * sizeof(param)); 
 
  /* clear them out with nochange values */ 
  for (i=0; i < numcells; i++)
    p->cells[index][i] = nochange; 

  if (p->num_tracks >= p->max_tracks) {  /* high water mark reached */ 
    p->max_tracks *= 2; 
    temp = realloc(p->cells, p->max_tracks * sizeof(param*));
    if (temp) p->cells = temp;
    else ox_die("add_track: allocation failed in pattern add track\n");
  }
}


void delete_track(pattern* p)
{
  int i = p->num_tracks--;

  free(p->cells[i]);
  p->cells[i] = 0; 
}

/* add bounds-checking to these? */ 

void set_cell(pattern* p, int track, int row, int col, param value)
{
  int offset = row * p->params_per_track + col; 

  p->cells[track][offset] = value; 
}


param get_cell(pattern* p, int track, int row, int col)
{
  int offset = row * p->params_per_track + col; 

  return p->cells[track][offset];
}


/* End $Source: /home/dto/octal/RCS/pattern.c,v $*/
